# -*- coding: utf-8 -*-
"""
*------------------------------
｜@Project   : ran_redis_lock 
｜@ Author   : ran <|-<
｜@ Time     : 2024/8/7 1:36
｜@ File     : lock.py
｜@ Software : PyCharm
*------------------------------
"""
# lock.py
import asyncio
from contextlib import asynccontextmanager
import aioredis
import os


class RedisDistributedLock:
    def __init__(self, key: str, redis_client: aioredis.Redis, timeout: int = 10, lock_timeout: int = 60):
        """
        初始化分布式锁。

        :param key: 锁的键名。
        :param redis_client: 已经建立好的 Redis 客户端实例。
        :param timeout: 获取锁的超时时间（秒）。
        :param lock_timeout: 锁的有效时间（秒）。
        """
        self.key = key
        self.redis_client = redis_client
        self.timeout = timeout
        self.lock_timeout = lock_timeout
        self.locked = False
        self.lock_value = None

    async def acquire(self) -> bool:
        """
        尝试获取锁。

        :return: 如果成功获取锁，则返回 True；否则返回 False。
        """
        lua_script = self._load_lua_script('acquire')
        script = self.redis_client.register_script(lua_script)
        lock_value = f"{self.key}-{id(self)}"
        acquired = await script(keys=[self.key], args=[lock_value, self.lock_timeout * 1000])
        if acquired:
            self.locked = True
            self.lock_value = lock_value
            return True
        return False

    async def release(self) -> bool:
        """
        释放锁。

        :return: 如果成功释放锁，则返回 True；否则返回 False。
        """
        if not self.locked:
            return False

        lua_script = self._load_lua_script('release')
        script = self.redis_client.register_script(lua_script)
        released = await script(keys=[self.key], args=[self.lock_value])
        if released:
            self.locked = False
            self.lock_value = None
            return True
        return False

    async def __aenter__(self) -> 'RedisDistributedLock':
        """
        异步上下文管理器的进入阶段。
        """
        start_time = asyncio.get_running_loop().time()
        while asyncio.get_running_loop().time() - start_time < self.timeout:
            if await self.acquire():
                return self
            await asyncio.sleep(0.1)
        raise TimeoutError(f"Failed to acquire lock for key {self.key} within {self.timeout} seconds.")

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        """
        异步上下文管理器的退出阶段。
        """
        await self.release()

    @asynccontextmanager
    async def lock(self):
        """
        上下文管理器，用于锁定和解锁。
        """
        try:
            yield
        finally:
            await self.release()

    @staticmethod
    def _load_lua_script(name: str) -> str:
        """
        加载 Lua 脚本。

        :param name: 脚本名称。
        :return: Lua 脚本字符串。
        """
        script_path = os.path.join(os.path.dirname(__file__), 'lua_scripts.lua')
        with open(script_path, 'r') as file:
            lua_content = file.read()
            scripts = eval(lua_content)
            return scripts[name]
